import warning from 'warning';
import { defineComponent, getCurrentInstance, ref, onMounted, h as _h, createCommentVNode, onUpdated } from 'vue-demi';
import { debug } from '../env';
import { COMPONENT_NAME } from '../shared/context';
import { pushSuspenseInstance, popSuspenseInstance } from '../shared/suspenseInstance';
import { FragmentComponent as Fragment } from '../shared/fragment';
export var del = function (asyncFactory, error) {
    var suspIns = asyncFactory.suspenseInstance;
    if (!suspIns) {
        return warning(!debug, 'No Suspense instance');
    }
    var asyncFactorys = suspIns.asyncFactorys;
    if (error) {
        suspIns.reject(error);
        return;
    }
    asyncFactorys.delete(asyncFactory);
    if (asyncFactorys.size === 0) {
        suspIns.resolve();
    }
};
export var add = function (asyncFactory) {
    var suspIns = asyncFactory.suspenseInstance;
    if (!suspIns) {
        return warning(!debug, 'No Suspense instance');
    }
    var asyncFactorys = suspIns.asyncFactorys || (suspIns.asyncFactorys = new Set());
    asyncFactorys.add(asyncFactory);
    suspIns.fallback();
};
export var has = function (asyncFactory) {
    var suspIns = asyncFactory.suspenseInstance;
    if (!suspIns) {
        return warning(!debug, 'No Suspense instance');
    }
    return suspIns.asyncFactorys && suspIns.asyncFactorys.has(asyncFactory);
};
export var suspenseOptions = {
    mode: 'visible',
    hiddleWrapperTag: 'div',
};
export function setSuspenseOptions(options) {
    Object.assign(suspenseOptions, options);
}
/**
 * Suspense component,
 * Vue2 compatible
 */
export var Suspense = defineComponent({
    name: COMPONENT_NAME,
    props: {
        timeout: {
            type: Number,
            default: 0,
        },
    },
    emits: ['pending', 'resolve', 'fallback'],
    setup: function (props, _a) {
        var slots = _a.slots, emit = _a.emit;
        var resolved = ref(false);
        var rejected = ref(false);
        var displayFallback = ref(false);
        var _time;
        var setupFallback = function () {
            if (resolved.value)
                return;
            if (!displayFallback.value) {
                emit('pending');
                if (props.timeout > 0) {
                    _time = setTimeout(function () {
                        displayFallback.value = true;
                        emit('fallback');
                    }, props.timeout);
                }
                else if (props.timeout === 0) {
                    displayFallback.value = true;
                    emit('fallback');
                }
            }
        };
        var destoryFallback = function () {
            if (_time) {
                clearTimeout(_time);
                _time = null;
            }
            displayFallback.value = false;
        };
        var onResolve = function () {
            destoryFallback();
            emit('resolve');
            resolved.value = true;
        };
        var onReject = function (err) {
            destoryFallback();
            emit('fallback');
            rejected.value = true;
        };
        var currPublicInstance = getCurrentInstance().proxy;
        currPublicInstance.resolve = onResolve;
        currPublicInstance.reject = onReject;
        currPublicInstance.fallback = setupFallback;
        pushSuspenseInstance(currPublicInstance);
        onMounted(function () {
            if (!currPublicInstance.asyncFactorys) {
                onResolve();
            }
        });
        onUpdated(function () {
            if (!resolved.value)
                return;
            popSuspenseInstance();
        });
        function createWrapper(children) {
            return _h(Fragment, children);
        }
        function createHiddenWrapper(children, display) {
            return _h(suspenseOptions.hiddleWrapperTag, {
                style: { display: display ? 'block' : 'none' },
                class: { 'suspense-hidden-wrapper': true },
            }, children);
        }
        return function () {
            var _a, _b;
            var isVisible = suspenseOptions.mode === 'visible';
            var fallback = displayFallback.value
                ? ((_a = slots.fallback) === null || _a === void 0 ? void 0 : _a.call(slots)) || (debug ? [createCommentVNode('no fallback')] : [])
                : [];
            // The `children` is the real content to be rendered
            var children = ((_b = slots.default) === null || _b === void 0 ? void 0 : _b.call(slots)) || (debug ? [createCommentVNode('no default slot')] : []);
            var rendered;
            if (rejected.value && slots.error) {
                rendered = createWrapper(slots.error);
            }
            else {
                rendered = isVisible
                    ? createWrapper(resolved.value ? children : children.concat(fallback))
                    : createWrapper([
                        // We need to render the children, but we should not show the rendered content.
                        createHiddenWrapper(children, resolved.value),
                        fallback,
                    ]);
            }
            return rendered;
        };
    },
});
//# sourceMappingURL=Suspense.js.map